package com.gct.edge.actuator.controller;

import com.gct.common.core.result.Result;
import com.gct.edge.actuator.service.DiagnoseService;
import com.gct.edge.base.entity.TableNameEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

/**
 * dec: Production
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/10/24 下午1:53
 */
@RestController
@RequestMapping("/module/diagnose")
public class DiagnoseController {

    @Autowired
    DiagnoseService diagnoseService;

    @PreAuthorize("hasAuthority('module:view:read')")
    @GetMapping("/getDiagnoseResultByDate")
    public Result getDiagnoseResultByDate(@RequestParam String moduleKey,
                                          @RequestParam String date,
                                          @RequestParam String wellId
    ) {
        return diagnoseService.getByDate(moduleKey, wellId, date);
    }

    @PreAuthorize("hasAuthority('module:view:write')")
    @GetMapping("/deleteResultByTimeRange")
    public Result deleteResultByTimeRange(@RequestParam String moduleKey,
                                          @RequestParam String beginTime,
                                          @RequestParam String endTime
    ) {
        return diagnoseService.deleteResultByTimeRange(moduleKey,beginTime,endTime);
    }

    @PreAuthorize("hasAuthority('module:view:write')")
    @PostMapping("/uploadDiagnoseResult")
    public Result uploadDiagnoseResult(@RequestParam MultipartFile file, @RequestParam String moduleKey) {
        try {
            return diagnoseService.loadDataFromFile(file.getInputStream(), moduleKey, TableNameEnum.DIAGNOSE_RESULT_REAL_TIME);
        } catch (Exception e) {
            return Result.error(file.getOriginalFilename() + " upload happened a error cause by: " + e.getMessage());
        }
    }
}
