package com.gct.edge.actuator.controller;

import com.gct.common.core.result.Result;
import com.gct.edge.actuator.config.SyncModuleDataProperties;
import com.gct.edge.actuator.service.DiagnoseService;
import com.gct.edge.actuator.service.YieldService;
import com.gct.edge.actuator.util.SshUtil;
import com.gct.edge.base.entity.TableNameEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Arrays;
import java.util.Date;
import java.util.stream.Collectors;

import static com.gct.edge.base.entity.TableNameEnum.*;

/**
 * @author xusirui 2021/11/1
 */
@RestController
@RequestMapping("/module/fileUpload")
public class FileUploadController {

    @Autowired
    SyncModuleDataProperties properties;

    @Autowired
    YieldService yieldService;

    @Autowired
    DiagnoseService diagnoseService;

    //    @PreAuthorize("hasAuthority('module:view:read')")
    @GetMapping("/loadRemoteFile")
    public Result loadRemoteFile(@RequestParam String moduleKey, @RequestParam String ipAddress, @RequestParam Integer port,
                                 @RequestParam String authUser, @RequestParam String authPassword, @RequestParam String remotePath) {
        for (int i = 0; i < values().length; i++) {
            TableNameEnum name = values()[i];
            try {
                File remoteFile = SshUtil.getRemoteFile(authUser, authPassword, ipAddress, port, remotePath + "/" + name.getActualFileName(), properties.getPath() + moduleKey + "/" + name.getActualFileName() + "-" + new Date().getTime());
                switch (name) {
                    case DIAGNOSE_RESULT_REAL_TIME:
                        diagnoseService.loadDataFromFile(new FileInputStream(remoteFile), moduleKey, DIAGNOSE_RESULT_REAL_TIME);
                        break;
                    case YIELD_REAL_TIME:
                        yieldService.loadDataFromFile(new FileInputStream(remoteFile), moduleKey, YIELD_REAL_TIME);
                        break;
                }
                if (remoteFile.exists()) remoteFile.delete();
            } catch (Exception e) {
                return Result.error("load data from remote file: " + name.getActualFileName() + " failed, cause by " + e.getMessage());
            }
        }
        return Result.ok("导入成功");
    }

    @PreAuthorize("hasAuthority('module:view:write')")
    @GetMapping("/loadLocalFile")
    public Result loadLocalFile(@RequestParam String path, @RequestParam String moduleKey, @RequestParam String uploadChoose) {
        try {
            if (DIAGNOSE_RESULT_REAL_TIME.getLabel().equals(uploadChoose)) {
                return diagnoseService.loadDataFromFile(new FileInputStream(path), moduleKey, DIAGNOSE_RESULT_REAL_TIME);
            } else if (YIELD_REAL_TIME.getLabel().equals(uploadChoose)) {
                return yieldService.loadDataFromFile(new FileInputStream(path), moduleKey, YIELD_REAL_TIME);
            } else {
                return Result.error("暂不支持该类型的文件");
            }
        } catch (Exception e) {
            return Result.error(path + " upload happened a error cause by: " + e.getMessage());
        }
    }

    @PreAuthorize("hasAuthority('module:view:read')")
    @GetMapping("/getUploadType")
    public Result getUploadType() {
        return Result.ok(Arrays.stream(values()).map(TableNameEnum::getLabel).collect(Collectors.toList()));
    }

    @PreAuthorize("hasAuthority('module:view:write')")
    @PostMapping("/upload")
    public Result upload(@RequestParam MultipartFile file, @RequestParam String moduleKey, @RequestParam String uploadChoose) {
        try {
            if (DIAGNOSE_RESULT_REAL_TIME.getLabel().equals(uploadChoose)) {
                return diagnoseService.loadDataFromFile(file.getInputStream(), moduleKey, DIAGNOSE_RESULT_REAL_TIME);
            } else if (YIELD_REAL_TIME.getLabel().equals(uploadChoose)) {
                return yieldService.loadDataFromFile(file.getInputStream(), moduleKey, YIELD_REAL_TIME);
            } else {
                return Result.error("暂不支持该类型的文件");
            }
        } catch (Exception e) {
            return Result.error(file.getOriginalFilename() + " upload happened a error cause by: " + e.getMessage());
        }
    }

    public static void main(String[] args) {
        String str = "0.110014,983265178430415874600798464027639284663149857225728858782857894139208394988737689584971761225685990413759667797908043493055607024962588709338661012551211469725946602827621051882192354710865657570726960784392876090720835533998436830355438683509011649843429376,970442281031356658117051822561946368617775443990699699466448647700328367889054481776040448413137315867084391449457889272501870104129569611753474933812714968096044871265310377303393085112381521173488602332258915834450226073486651575427871618050459543858577408,970464275709567753898951633430701595335794742928379995585721771606188804228210404399967706905263778668320144315459399654343999159828185191640004330150175462712342821845270705698627981603477303577229951947375568429844418375121928862723585807573047854069972992,970471607268974297512409599124776944978565216977600209389030334362290718744595397930498261925656448746471100467290620098386562043243647297490886553191763249283024466526077149097606873080054175291879057526398336986716033787283866813207986527370728187394785280,970486270387785677727630713270233479822047740434044567937516947002005486335365573398026490652631479416129589653054630796414575891165017361659427544717478249405238359195198235535224300445480952173136075260053543485415133824637635401345290372991998316091801600,985493972489482787513405958762131256795496027737876723200724033012537606717833375245198805399102991258762863877679420324518476396939499742840479722610963243353451197890288934907626553745782895123449868943873349176187900018374736051174993745932128367088238592,985501304048889331126863924456206606438266501787096937004032595768639521234218368775729360419495661336913820029510640768561039280354961848691361945652551029924132842571095378306605445222359766838098974522896117733059515430536674001659394465729808700413050880,985515967167700711342085038601663141281749025243541295552519208408354288824988544243257589146470692006572309215274651466589053128276331912859902937178266030046346735240216464744222872587786543719355992256551324231758615467890442589796698311351078829110067200,985537961845911807123984849470418367999768324181221591671792332314214725164144466867184847638597154807808062081276161848431182183974947492746432333515726524662644685820176793139457769078882326123097341871667976827152807769525719877092412500873667139321462784,985545293405318350737442815164493717642538798230441805475100895070316639680529460397715402658989824885959018233107382292473745067390409598597314556557314311233326330500983236538436660555459197837746447450690745384024423181687657827576813220671347472646275072,985559956524129730952663929309950252486021321686886164023587507710031407271299635865243631385964855555617507418871392990501758915311779662765855548083029311355540223170104322976054087920885974719003465184345951882723523219041426415714117066292617601343291392,985581951202340826734563740178705479204040620624566460142860631615891843610455558489170889878091318356853260284872903372343887971010395242652384944420489805971838173750064651371288984411981757122744814799462604478117715520676703703009831255815205911554686976,985589282761747370348021705872780828846811094673786673946169194371993758126840552019701444898483988435004216436704123816386450854425857348503267167462077592542519818430871094770267875888558628837393920378485373034989330932838641653494231975612886244879499264,985603945880558750563242820018237363690293618130231032494655807011708525717610727487229673625459019104662705622468134514414464702347227412671808158987792592664733711099992181207885303253985405718650938112140579533688430970192410241631535821234156373576515584,985625940558769846345142630886992590408312917067911328613928930917568962056766650111156932117585481905898458488469644896256593758045842992558337555325253087281031661679952509603120199745081188122392287727257232129082623271827687528927250010756744683787911168,985633272118176389958600596581067940051083391117131542417237493673670876573151643641687487137978151984049414640300865340299156641461305098409219778366840873851713306360758953002099091221658059837041393306280000685954238683989625479411650730554425017112723456,985647935236987770173821710726524474894565914573575900965724106313385644163921819109215715864953182653707903826064876038327170489382675162577760769892555873973927199029880039439716518587084836718298411039935207184653338721343394067548954576175695145809739776,985669929915198865955721521595279701612585213511256197084997230219246080503077741733142974357079645454943656692066386420169299545081290742464290166230016368590225149609840367834951415078180619122039760655051859780047531022978671354844668765698283456021135360,985677261474605409569179487289355051255355687560476410888305792975347995019462735263673529377472315533094612843897606864211862428496752848315172389271604155160906794290646811233930306554757490836688866234074628336919146435140609305329069485495963789345947648,985691924593416789784400601434811586098838211016920769436792405615062762610232910731201758104447346202753102029661617562239876276418122912483713380797319155283120686959767897671547733920184267717945883967729834835618246472494377893466373331117233918042963968,985713919271627885566300412303566812816857509954601065556065529520923198949388833355129016596573809003988854895663127944082005332116738492370242777134779649899418637539728226066782630411280050121687233582846487431012438774129655180762087520639822228254359552,985721250831034429179758377997642162459627984003821279359374092277025113465773826885659571616966479082139811047494348388124568215532200598221125000176367436470100282220534669465761521887856921836336339161869255987884054186291593131246488240437502561579171840,985735913949845809394979492143098697303110507460265637907860704916739881056544002353187800343941509751798300233258359086152582063453570662389665991702082436592314174889655755903378949253283698717593356895524462486583154223645361719383792086058772690276188160,985757908628056905176879303011853924021129806397945934027133828822600317395699924977115058836067972553034053099259869467994711119152186242276195388039542931208612125469616084298613845744379481121334706510641115081977346525280639006679506275581361000487583744,985765240187463448790337268705929273663900280447166147830442391578702231912084918507645613856460642631185009251091089912037274002567648348127077611081130717779293770150422527697592737220956352835983812089663883638848961937442576957163906995379041333812396032,985779903306274829005558382851385808507382803903610506378929004218416999502855093975173842583435673300843498436855100610065287850489018412295618602606845717901507662819543614135210164586383129717240829823319090137548061974796345545301210841000311462509412352,985801897984485924787458193720141035225402102841290802498202128124277435842011016599101101075562136102079251302856610991907416906187633992182147998944306212517805613399503942530445061077478912120982179438435742732942254276431622832596925030522899772720807936,985809229543892468400916159414216384868172576890511016301510690880379350358396010129631656095954806180230207454687831435949979789603096098033030221985893999088487258080310385929423952554055783835631285017458511289813869688593560783081325750320580106045620224,985823892662703848616137273559672919711655100346955374849997303520094117949166185597159884822929836849888696640451842133977993637524466162201571213511608999210701150749431472367041379919482560716888302751113717788512969725947329371218629595941850234742636544,985845887340914944398037084428428146429674399284635670969270427425954554288322108221087143315056299651124449506453352515820122693223081742088100609849069493826999101329391800762276276410578343120629652366230370383907162027582606658514343785464438544954032128,985853218900321488011495050122503496072444873333855884772578990182056468804707101751617698335448969729275405658284572959862685576638543847938982832890657280397680746010198244161255167887155214835278757945253138940778777439744544608998744505262118878278844416,985867882019132868226716164267960030915927396790300243321065602821771236395477277219145927062424000398933894844048583657890699424559913912107523824416372280519894638679319330598872595252581991716535775678908345439477877477098313197136048350883389006975860736,985889876697343964008615975136715257633946695727980539440338726727631672734633199843073185554550463200169647710050094039732828480258529491994053220753832775136192589259279658994107491743677774120277125294024998034872069778733590484431762540405977317187256320,985897208256750507622073940830790607276717169777200753243647289483733587251018193373603740574943133278320603861881314483775391363673991597844935443795420561706874233940086102393086383220254645834926230873047766591743685190895528434916163260203657650512068608,985911871375561887837295054976247142120199693233645111792133902123448354841788368841131969301918163947979093047645325181803405211595361662013476435321135561829088126609207188830703810585681422716183248606702973090442785228249297023053467105824927779209084928,985933866053772983619194865845002368838218992171325407911407026029308791180944291465059227794044626749214845913646835563645534267293977241900005831658596056445386077189167517225938707076777205119924598221819625685836977529884574310349181295347516089420480512,985941197613179527232652831539077718480989466220545621714715588785410705697329284995589782814437296827365802065478056007688097150709439347750888054700183843016067721869973960624917598553354076834573703800842394242708592942046512260833582015145196422745292800,985955860731990907447873945684534253324471989676989980263202201425125473288099460463118011541412327497024291251242066705716110998630809411919429046225898843138281614539095047062535025918780853715830721534497600741407692979400280848970885860766466551442309120,985977855410202003229773756553289480042491288614670276382475325330985909627255383087045270033538790298260044117243577087558240054329424991805958442563359337754579565119055375457769922409876636119572071149614253336801885281035558136266600050289054861653704704,985985186969608546843231722247364829685261762663890490185783888087087824143640376617575825053931460376411000269074797531600802937744887097656840665604947124325261209799861818856748813886453507834221176728637021893673500693197496086751000770086735194978516992,985999850088419927058452836392821364528744286120334848734270500726802591734410552085104053780906491046069489454838808229628816785666257161825381657130662124447475102468982905294366241251880284715478194462292228392372600730551264674888304615708005323675533312,986021844766631022840352647261576591246763585058015144853543624632663028073566474709031312273032953847305242320840318611470945841364872741711911053468122619063773053048943233689601137742976067119219544077408880987766793032186541962184018805230593633886928896,986029176326037566453810612955651940889534059107235358656852187388764942589951468239561867293425623925456198472671539055513508724780334847562793276509710405634454697729749677088580029219552938833868649656431649544638408444348479912668419525028273967211741184,986043839444848946669031727101108475733016582563679717205338800028479710180721643707090096020400654595114687658435549753541522572701704911731334268035425405756668590398870763526197456584979715715125667390086856043337508481702248500805723370649544095908757504,986065834123060042450931537969863702451035881501360013324611923934340146519877566331017354512527117396350440524437060135383651628400320491617863664372885900372966540978831091921432353076075498118867017005203508638731700783337525788101437560172132406120153088,986073165682466586064389503663939052093806355550580227127920486690442061036262559861547909532919787474501396676268280579426214511815782597468745887414473686943648185659637535320411244552652369833516122584226277195603316195499463738585838279969812739444965376,986087828801277966279610617809395586937288879007024585676407099330156828627032735329076138259894818144159885862032291277454228359737152661637286878940188687065862078328758621758028671918079146714773140317881483694302416232853232326723142125591082868141981696";
        Arrays.stream(str.split(",")).forEach(System.out::println);
    }

}
