package com.gct.edge.actuator.controller;

import com.gct.common.core.result.Result;
import com.gct.edge.actuator.entity.DiagnoseResultRealTime;
import com.gct.edge.actuator.entity.YieldRealTime;
import com.gct.edge.actuator.service.YieldService;
import com.gct.edge.base.entity.TableNameEnum;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.lang.reflect.Field;

import static com.gct.edge.actuator.util.SqlGeneratorUtil.camelToUnderline;

/**
 * dec: Production
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/10/24 下午1:53
 */
@RestController
@RequestMapping("/module/yield")
public class YieldController {

    @Autowired
    YieldService yieldService;

    @PreAuthorize("hasAuthority('module:view:write')")
    @PostMapping("/uploadYieldReuslt")
    public Result uploadDiagnoseReuslt(@RequestParam MultipartFile file, @RequestParam String moduleKey) {
        try {
            return yieldService.loadDataFromFile(file.getInputStream(), moduleKey, TableNameEnum.YIELD_REAL_TIME);
        } catch (Exception e) {
            return Result.error(file.getOriginalFilename() + " upload happened a error cause by: " + e.getMessage());
        }
    }

    @PreAuthorize("hasAuthority('module:view:read')")
    @GetMapping("/getProdCurve")
    public Result getProdCurve(@RequestParam String moduleKey,
                               @RequestParam String date,
                               @RequestParam String wellId) {
        return Result.ok(yieldService.getProdCurve(moduleKey, date, wellId));
    }

    public static void main(String[] args) {
        Class<YieldRealTime> claz = YieldRealTime.class;
        Field[] declaredFields = claz.getDeclaredFields();
        String v1 = "";
        String v2 = "";
        for (int i = 0; i < declaredFields.length; i++) {
            ApiModelProperty[] an = declaredFields[i].getAnnotationsByType(ApiModelProperty.class);
            if ( an== null||an.length==0) continue;
            String name = declaredFields[i].getName();
            String s_ = camelToUnderline(name);

            v1 += "#{" + name + "}";
            if (i != declaredFields.length - 1)
                v1 += ",";

            v2 += s_;
            if (i != declaredFields.length - 1)
                v2 += ",";
        }
        System.out.println("v2 = " + v2);
        System.out.println("v1 = " + v1);
    }
}
