package com.gct.edge.actuator.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.gct.edge.actuator.exception.ObjectParseException;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.lang.reflect.InvocationTargetException;
import java.time.LocalDateTime;

/**
 * @author xusirui 2021/10/26
 */
@Data
public class YieldRealTime extends Model<YieldRealTime> implements FileParseObject {

    private static final long serialVersionUID = 1L;
    /**
     * shard key
     */
    @ApiModelProperty(value = "id")
    @TableId(type = IdType.ASSIGN_UUID)
    private String id;

    @ApiModelProperty(value = "分库分表所需key")
    private String moduleKey;


    @ApiModelProperty(value = "井号")
    private String wellId;

    @ApiModelProperty(value = "井名称")
    private String wellCommonName;


    @ApiModelProperty(value = "实时时间")
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private LocalDateTime proTime;

    @ApiModelProperty(value = "冲程")
    private Double stroke;

    @ApiModelProperty(value = "冲次")
    private Double frequence;

    @ApiModelProperty(value = "泵径")
    private Double pumpDiameter;

    @ApiModelProperty(value = "泵深")
    private Double pumpDepth;

    @ApiModelProperty(value = "沉没度")
    private Double submergence;

    @ApiModelProperty(value = "油嘴直径")
    private Double nozzleDiameter;

    @ApiModelProperty(value = "上行载荷")
    private Double riseLoad;

    @ApiModelProperty(value = "下行载荷")
    private Double declineLoad;

    @ApiModelProperty(value = "示功图")
    private String workDiagram;

    @ApiModelProperty(value = "泵功图")
    private String pumpWorkDiagram;

    @ApiModelProperty(value = "一次诊断结果")
    private String result1;

    @ApiModelProperty(value = "参考井号")
    private String refrenceWellId;

    @ApiModelProperty(value = "参考创建时间")
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private LocalDateTime refrenceCreateTime;

    @ApiModelProperty(value = "日产液")
    private Double liqProdDaily;

    @ApiModelProperty(value = "日产油")
    private Double oilProdDaily;

    @ApiModelProperty(value = "日产气")
    private Double gasProdDaily;

    @ApiModelProperty(value = "二次诊断结果")
    private String result2;

    @ApiModelProperty(value = "泵效")
    private Double pumpEfficiency;

    @ApiModelProperty(value = "含水率")
    private Double moistureContent;

    @ApiModelProperty(value = "标记")
    private Double flag;

    @ApiModelProperty(value = "膨胀系数")
    private Double expandCoefficient;

    @ApiModelProperty(value = "理论排量")
    private Double theoryDisplacement;

    @ApiModelProperty(value = "左冲程")
    private Double leftStroke;

    @ApiModelProperty(value = "右冲程")
    private Double rightStroke;

    @ApiModelProperty(value = "实际冲程")
    private Double actualStroke;

    @ApiModelProperty(value = "有效冲程")
    private Double effectStroke;

    @ApiModelProperty(value = "冲程损失")
    private Double strokeLoss;

    @ApiModelProperty(value = "漏失损失")
    private Double outflowLoss;

    @ApiModelProperty(value = "供液损失")
    private Double liqLoss;

    @ApiModelProperty(value = "回压")
    private Double backPress;

    @ApiModelProperty(value = "油压")
    private Double oilPress;

    @ApiModelProperty(value = "套压")
    private Double sleevePress;

    @ApiModelProperty(value = "日产水")
    private Double waterProdDaily;

    @ApiModelProperty(value = "参考功图")
    private String refrenceWorkDiagram;

    @ApiModelProperty(value = "以此功图生产此时间的产出量（0-24h）")
    private Double presentTime;

    @ApiModelProperty(value = "气液比")
    private Double gasLiqRatio;

    @ApiModelProperty(value = "有效冲程比")
    private Double effectStrokeRatio;

    @ApiModelProperty(value = "电压")
    private Double voltage;

    @ApiModelProperty(value = "环境温度")
    private Double ambientTemperature;

    @ApiModelProperty(value = "电量")
    private Double electricity;

    @ApiModelProperty(value = "信号强度")
    private Double signalStrength;

    @ApiModelProperty(value = "设备号")
    private Double deviceId;

    @Override
    public void initFromTxtRow(String[] objFields) throws ObjectParseException, NoSuchMethodException, InvocationTargetException, IllegalAccessException, InstantiationException {
        //记录txt文件中的数据对应的类名及类型 与单列数据split后的含义一致
         final String[] colName = {
                "wellId", "wellCommonName", "proTime", "stroke", "frequence", "pumpDiameter",
                "pumpDepth", "submergence", "nozzleDiameter", "riseLoad", "declineLoad", "workDiagram",
                "pumpWorkDiagram", "result1", "refrenceWellId", "refrenceCreateTime", "liqProdDaily", "oilProdDaily",
                "gasProdDaily", "result2", "pumpEfficiency", "moistureContent", "flag", "expandCoefficient",
                "theoryDisplacement", "leftStroke", "rightStroke", "actualStroke", "effectStroke", "strokeLoss",
                "outflowLoss", "liqLoss", null, /*数据重复有两个泵深*/"backPress", "oilPress", "sleevePress",
                "waterProdDaily", "refrenceWorkDiagram", "presentTime", "gasLiqRatio", "effectStrokeRatio", "voltage",
                "ambientTemperature", "electricity", "signalStrength", "deviceId"
        };
        final String[] colType = {
                "String", "String", "LocalDateTime", "Double", "Double", "Double",
                "Double", "Double", "Double", "Double", "Double", "String",
                "String", "String", "String", "LocalDateTime", "Double", "Double",
                "Double", "String", "Double", "Double", "Double", "Double",
                "Double", "Double", "Double", "Double", "Double", "Double",
                "Double", "Double", null,/*数据重复有两个泵深*/"Double", "Double", "Double",
                "Double", "String", "Double", "Double", "Double", "Double",
                "Double", "Double", "Double", "Double"
        };
        initFromTxtRow(objFields,colName,colType);
    }


}
