package com.gct.edge.actuator.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gct.common.core.result.Result;
import com.gct.common.util.DateTimeUtil;
import com.gct.common.util.SGTUtil;
import com.gct.common.util.StringUtil;
import com.gct.edge.actuator.entity.DiagnoseResultRealTime;
import com.gct.edge.actuator.mapper.DiagnoseResultRealTimeMapper;
import com.gct.edge.actuator.service.DiagnoseService;
import com.gct.edge.actuator.service.ServicePlusImpl;
import com.gct.edge.actuator.util.DataTranslator;
import com.gct.edge.base.entity.TableNameEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.InputStream;
import java.time.LocalDateTime;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * dec: DiagnoseServiceImpl
 *
 * @author lloyd
 * @version 1.0
 * @since 2021/10/24 下午2:08
 */
@Service
public class DiagnoseServiceImpl extends ServicePlusImpl<DiagnoseResultRealTimeMapper, DiagnoseResultRealTime> implements DiagnoseService {

    @Autowired
    DiagnoseResultRealTimeMapper mapper;

    @Override
    public Result getByDate(String moduleKey, String wellId, String date) {
        if (StringUtil.isBlankOrEmpty(moduleKey)) return Result.error("module key must be not null");
        LocalDateTime curDay = null;
        try {
            curDay = DateTimeUtil.dateToDateTime(DateTimeUtil.formatDate(date));
        } catch (Exception e) {
            return Result.error(e.getMessage());
        }
        List<DiagnoseResultRealTime> list = mapper.getListByDateRange(moduleKey, wellId, curDay, curDay.plusDays(1));
        List<JSONObject> res = list.stream().map(x -> {
            System.out.println("x 111111111111111111111 = " + x);
            if(x==null) x=new DiagnoseResultRealTime().setWorkDiagram("0,0,0,0,0,0,0,0,0,1");
            if (StringUtil.isBlankOrEmpty(x.getWorkDiagram())) {
                x.setWorkDiagram("0,0,0,0,0,0,0,0");
                System.out.println("x = " + x);

            }
            List<List<Double>> gt = SGTUtil.encodeToUseList(x.getWorkDiagram());
            JSONObject obj = (JSONObject) JSON.toJSON(x);
            obj.put("workDiagram", gt);
            obj.put("minS", gt.stream().min(Comparator.comparingDouble(p -> p.get(0))).get().get(0));
            obj.put("maxS", gt.stream().max(Comparator.comparingDouble(p -> p.get(0))).get().get(0));
            obj.put("minL", gt.stream().min(Comparator.comparingDouble(p -> p.get(1))).get().get(1));
            obj.put("maxL", gt.stream().max(Comparator.comparingDouble(p -> p.get(1))).get().get(1));
            obj.put("proTime", DateTimeUtil.formatString(x.getProTime()));
            return obj;
        }).collect(Collectors.toList());
        return Result.ok(res);
    }

    @Override
    public Result loadDataFromFile(InputStream file, String moduleKey, TableNameEnum tableKey) {

        boolean checkParams = true;
        if (StringUtil.isBlankOrEmpty(moduleKey)) checkParams = false;
        if (Objects.isNull(tableKey)) checkParams = false;
        if (!checkParams) return Result.error("input params is not supported");

        List<DiagnoseResultRealTime> data = DataTranslator.transFileToFileParseObject(file, DiagnoseResultRealTime.class);

        data.forEach(x -> {
            x.setModuleKey(moduleKey);
        });

        int sus =saveOnConflictDoNothingBatch(data);
        return sus > 0 ? Result.ok("导入了" + (sus) + "条") : Result.error("导入了0条,检查格式、重复？");
    }

    @Override
    public Result deleteResultByTimeRange(String moduleKey, String beginTime, String endTime) {
        int res = baseMapper.delete(new QueryWrapper<DiagnoseResultRealTime>()
                .eq("module_key",moduleKey)
                .between("pro_time",DateTimeUtil.formatDateTime(beginTime),DateTimeUtil.formatDateTime(endTime)));
        return res>0?Result.ok("删除成功,共"+res+"条"):Result.error("删除失败");
    }
}
