package com.gct.edge.actuator.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.enums.SqlMethod;
import com.baomidou.mybatisplus.core.metadata.TableInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfoHelper;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.core.toolkit.ReflectionKit;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.common.core.result.Result;
import com.gct.common.util.DateTimeUtil;
import com.gct.common.util.StringUtil;
import com.gct.edge.actuator.entity.DiagnoseResultRealTime;
import com.gct.edge.actuator.entity.YieldRealTime;
import com.gct.edge.actuator.mapper.YieldRealTimeMapper;
import com.gct.edge.actuator.service.ServicePlusImpl;
import com.gct.edge.actuator.service.YieldService;
import com.gct.edge.actuator.util.DataTranslator;
import com.gct.edge.base.entity.TableNameEnum;
import io.swagger.models.auth.In;
import org.apache.ibatis.binding.MapperMethod;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.time.ZoneOffset;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author xusirui 2021/10/26
 */
@Service
public class YieldServiceImpl extends ServicePlusImpl<YieldRealTimeMapper, YieldRealTime> implements YieldService {


    @Override
    public Result loadDataFromFile(InputStream file, String moduleKey, TableNameEnum tableKey) throws IOException {
        boolean checkParams = !StringUtil.isBlankOrEmpty(moduleKey);
        if (Objects.isNull(tableKey)) checkParams = false;
        if (!checkParams) return Result.error("input params is not supported");

        List<YieldRealTime> data = DataTranslator.transFileToFileParseObject(file, YieldRealTime.class);
        data.forEach(x -> {
            x.setModuleKey(moduleKey);
        });
        int sus = saveOnConflictDoNothingBatch(data);
//        int failed = 0;
//        for (int i = 0; i < data.size(); i++) {
//            try{
//                this.save(data.get(i));
//            }catch (Exception e){
//                failed++;
//            }
//        }
        return sus > 0 ? Result.ok("导入了" + (sus) + "条") : Result.error("导入了0条,检查格式、重复？");
    }

    @Override
    public JSONObject getProdCurve(String moduleKey, String date, String wellId) {
        List<YieldRealTime> list = baseMapper.selectList(new QueryWrapper<YieldRealTime>()
                .eq("module_key", moduleKey)
                .between("pro_time", DateTimeUtil.formatDate(date).atTime(0, 0, 0),
                        DateTimeUtil.formatDate(date).plusDays(1).atTime(0, 0, 0))
                .eq("well_id", wellId)
                .orderByAsc("pro_time"));
        JSONObject obj = new JSONObject();
        obj.put("liqProdDailyArr", list.stream()
                .map(x -> new Object[]{x.getProTime().toEpochSecond(ZoneOffset.of("+8")) * 1000, x.getLiqProdDaily()})
                .collect(Collectors.toList()));
        obj.put("oilProdDailyArr", list.stream()
                .map(x -> new Object[]{x.getProTime().toEpochSecond(ZoneOffset.of("+8")) * 1000, x.getOilProdDaily()})
                .collect(Collectors.toList()));
        obj.put("proTimeArr", list.stream()
                .map(x -> x.getProTime().toEpochSecond(ZoneOffset.of("+8")))
                .collect(Collectors.toList()));
        obj.put("moistureContent", list.isEmpty() ? null : list.get(0).getMoistureContent());
        return obj;
    }
}
