package com.gct.edge.actuator.util;

import com.gct.edge.actuator.entity.YieldRealTime;
import io.swagger.annotations.ApiModelProperty;

import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.Map;

/**
 * @author xusirui 2021/10/26
 */
public class SqlGeneratorUtil {

    public static final char UNDERLINE='_';
    public static String camelToUnderline(String param){
        if (param==null||"".equals(param.trim())){
            return "";
        }
        int len=param.length();
        StringBuilder sb=new StringBuilder(len);
        sb.append(Character.toLowerCase(param.charAt(0)));
        for (int i = 1; i < len; i++) {
            char c=param.charAt(i);
            if (Character.isUpperCase(c)){
                sb.append(UNDERLINE);
                sb.append(Character.toLowerCase(c));
            }else{
                sb.append(c);
            }
        }
        return sb.toString();
    }

    public static String sqlGenerate(Class<?> clz){
        Map<String,String> typeMap = new HashMap<>();
        typeMap.put("LocalDateTime","timestamp");
        typeMap.put("String","varchar");
        typeMap.put("Double","double precision");
        typeMap.put("Integer","int");
        String[] tableNameArr = clz.getName().split("\\.");
        StringBuilder sql =new StringBuilder();
        String tableName = camelToUnderline(tableNameArr[tableNameArr.length - 1]);
        sql.append("create table ").append(tableName).append("(\n");
        Field[] fields = clz.getDeclaredFields();
        for (int i = 0; i < fields.length; i++) {
            Field field = fields[i];
            if (field.getAnnotation(ApiModelProperty.class)==null)continue;
            String[] split = field.getType().getName().split("\\.");
            String type = split[split.length - 1];
            String name = field.getName();
            sql.append(camelToUnderline(name)).append(" ").append(typeMap.get(type)).append(",\n");
        }
        sql.deleteCharAt(sql.length()-2);
        sql.append(");\n");

        for (int i = 0; i < fields.length; i++) {
            Field field = fields[i];
            ApiModelProperty annotation = field.getAnnotation(ApiModelProperty.class);
            if (annotation == null)continue;
            sql.append(String.format("comment on column %s.%s is '%s';\n",tableName,camelToUnderline(fields[i].getName()),annotation.value()));
        }
        return sql.toString();
    }

    public static void main(String[] args) {
        System.out.println(sqlGenerate(YieldRealTime.class));
    }
}
